package net.jieyuu.service.impl;

import io.minio.*;
import lombok.extern.slf4j.Slf4j;
import net.jieyuu.service.FileService;
import net.jieyuu.utils.CommonUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;


@Slf4j
@Service
public class FileServiceImpl implements FileService {

    @Autowired
    private MinioClient minioClient;

    @Value("${minio.bucketName}")
    private String bucketName;
    @Value("${minio.endpoint}")
    private String endpoint;

    @Override
    public String uploadUserHeadImg(MultipartFile file) {
        String originalFilename = file.getOriginalFilename();

        LocalDateTime localDateTime = LocalDateTime.now();

        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy/MM/dd");
        String folder = dateTimeFormatter.format(localDateTime);

        String fileName = CommonUtil.generateUUID();
        String extension = originalFilename.substring(originalFilename.lastIndexOf("."));
        String newFileName = "user/" + folder + "/" + fileName + extension;


        try (InputStream in = file.getInputStream();){

            ObjectWriteResponse objectWriteResponse = minioClient.putObject(PutObjectArgs.builder()
                    .bucket(bucketName)
                    .object(newFileName)
                    .stream(in, in.available(), -1)
                    .contentType(file.getContentType())
                    .build()
            );
            if (objectWriteResponse != null) {
                String res = endpoint + "/" + bucketName + "/" + newFileName;
                return res;
            }
        } catch (Exception e) {
            log.error("上传头像失败:{}", e);
        }
        return null;
    }
}